#include "atari.h"
#include "config.h"
#include "input.h"
#include "ui.h"
#include "atari.h"
#include "platform.h"

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <linux/input.h>
#include <unistd.h>
#include <fcntl.h>

#include "atari_gp32.h"
#include "sound.h"
#include "pokey.h"
#include "antic.h"
#include "menu_c.h"
#include "menu_keymap.h"
#include "gp2xjoy.h"
#include "pandorainput.h"

#include "gp2xminlib.h"

#include "log.h"
#include "util.h"

#include <math.h>

unsigned char menuWidth = 0; //Either height or width at present...
unsigned char menuHeight = 0;
int fastForward = 0;
static int brokenUI = 0;
static int brokenNormal = 0;
static int keymapInit = 0;

char atari_menu_filename[FILENAME_MAX] = FILENAME_NOT_SET;
char atari_uimenu_filename[FILENAME_MAX] = FILENAME_NOT_SET;

void setFastForward(int onOff)
{
	if (onOff == fastForward)
		return;
	if (onOff)
	{
		// Set to max known possible clock speed
		Sound_Pause();
		GP32_SetClockMax(1);
		fastForward = 1;
	}
	else
	{
		GP32_SetClockMax(0);
		Sound_Continue();
		fastForward = 0;
	}
}

///////////////////////////////////////////////////////////////////////
// Interface -> called externally
void Menu_Initialise()
{
	if (!keymapInit)
	{
		initKeymap();
		keymapInit = 1;
	}

	char * menuFile = malloc(FILENAME_MAX);

	brokenUI = 0;
	brokenNormal = 0;

	menuActivateUi();
	if (!menuLoad(atari_uimenu_filename))
	{
		brokenUI = 1;
	}

	menuActivateMain();
	if (!menuLoad(atari_menu_filename))
	{
		brokenNormal = 1;
	}

	free(menuFile);

	// Init chatboard listener
	//TODO preInitKbd(9600, GpPClkGet());
}

void Menu_Frame()
{
	if (brokenUI || brokenNormal)
	{
		GpRectFill(0, Atari_DisplayActiveBuffer(), 0, 0,336,240,0);
		GpTextOut(0,Atari_DisplayActiveBuffer(),20,50,"Unable to load input cfg, need", 0x48);
		GpTextOut(0,Atari_DisplayActiveBuffer(),20,60,"ui.men(UIMENU in cfg)", 0x0f);
		GpTextOut(0,Atari_DisplayActiveBuffer(),20,90,"default.men(MENU in cfg)", 0x0f);
		GpTextOut(0,Atari_DisplayActiveBuffer(),20,150,"Used built in defaults", 0x0f);
		gp2x_video_flip();
		while (!joyPressed()) usleep(1);
		while (joyPressed()) usleep(1);
		brokenUI = brokenNormal = 0;
	}
	
	int joyswitches,x,y,buttons;

	readAllJoy(&joyswitches, &x,&y,&buttons);

	menuFrame(x,y,buttons);

	int operation = menuOper();
	key_code = menuKey();
	key_shift = menuShift();
	key_consol = menuConsol();
	int key_5200power = menu5200Power();


	//if (key_5200power!=0)
	//	fprintf(stderr, "Power:%d\n",key_5200power);
	menuJoy(joyValue, fireValue);

	// mouse click
	// X,Y,YES
	// int keyX = -1;
	// int keyY = -1;
	struct MousePress mousePress;
	getMousePress(&mousePress);

	/* There should be UI or options for light pen/gun offsets.
	   Below are best offsets for different programs:
	   AtariGraphics: H = 0..32, V = 0 (there's calibration in the program)
	   Bug Hunt: H = 44, V = 2
	   Barnyard Blaster: H = 40, V = 0
	   Operation Blood (light gun version): H = 40, V = 4
	 */
	// keyX = 0->335
	// keyY = 0->239
	// X:0-167  (168*2=336 - perfect!)
	// Y:0-119
	if (mousePress.pressed_)
	{
		if (mousePress.x_>68 && mousePress.x_<(800-68))
		{
			mousePress.x_-=68;
			double temp = mousePress.x_;
			mousePress.x_ = (int)(temp*336.0/672.0);
		}
		double temp = mousePress.y_;
		mousePress.y_ = (int)(temp*240.0/480.0);

		// 336
		// 240
		int adjKeyX = mousePress.x_/2; 
		int adjKeyY = mousePress.y_/2;
		int mouse_pen_ofs_h = 42;
		int mouse_pen_ofs_v = 2;
		PENH_input = 44 + mouse_pen_ofs_h + adjKeyX;
		PENV_input = 4 + mouse_pen_ofs_v + adjKeyY;

		joyValue[0] = 14;
	}
	else
	{
		PENH_input = 0;
		PENV_input = 0;
	}

	// Not allowed to remap joy on 5200, so there!
	// Hardcoded...
	int i=0;
        if (machine_type == MACHINE_5200) 
	{
		if (!(key_consol&CONSOL_START))
			key_code = AKEY_5200_START;
		else if (!(key_consol&CONSOL_SELECT))
			key_code = AKEY_5200_PAUSE;
		else if (key_code == AKEY_WARMSTART)
			key_code = AKEY_5200_RESET;
		
#ifdef PANDORA
		int x5200_mid = 120;
		int y5200_mid = 114;

		// read nubs
		struct NubPosition leftNub;
		struct NubPosition rightNub;
		queryLeftNub(&leftNub);
		queryRightNub(&rightNub);
	
		//printf("leftNub: x:%d y:%d rightNub: x:%d y:%d\n",leftNub.x_,leftNub.y_,rightNub.x_,rightNub.y_);

		// store in POT_input
                for (i = 0; i < 4; i++) {
			if (i==0)
			{
				POT_input[2 * i] = x5200_mid + 100*leftNub.x_/256;
				POT_input[2 * i + 1] = y5200_mid + 100*leftNub.y_/256;
			}
			if (i==1)
			{
				POT_input[2 * i] = x5200_mid + 100*rightNub.x_/256;
				POT_input[2 * i + 1] = y5200_mid + 100*rightNub.y_/256;
			}
                }
#else
		int x5200_mid = 120;
		int y5200_mid = 114;
		int x5200 = x5200_mid;
		int y5200 = y5200_mid;
		if (key_5200power!=-1)//not digital)
		{
			// work out angle
			// yeah I know I have no fpu, but I'm too lazy for int maths here (and its only once per frame so it doesn't matter anyway)
			// only called once per frame
			double angle = 0;
			double length;
			if (joyswitches == 0)
			{
				angle = 0.0;
				length = 0.0;
			}
			else if ((joyswitches&(joyswitches-1)) == 0)
			{
				// single bit
				for (i=1; i!=joyswitches; i<<=1)angle-=M_PI_4;
				length = 1.0;
			}
			else
			{
				angle = -(M_PI/8.0);

				// two bits
				unsigned char j=0;
				for (j=3; j!=joyswitches; j&128 ? (j=(j<<1)|1) : (j<<=1))angle-=M_PI_4;
				length = 1.0;
			}

			// Adjust power if modifier keys pressed
			// Note they may be toggled to hold them on
			static int power = 0;
			if (power != key_5200power)
				fprintf(stderr, "POWER:%d\n", key_5200power);
			power =key_5200power;
			if (key_5200power!=0)
				length*=(double)key_5200power;
			else
				length*=40.0;// default...

			x5200 = (length*sin(angle) + x5200_mid ) + 0.5;
			y5200 = (-length*cos(angle) + y5200_mid) + 0.5;
		}
		else
		{
			x5200 += x*100;
			y5200 += y*100;
		}
		
                for (i = 0; i < 4; i++) {
			// x - from 2*i from min to max via center
                        POT_input[2 * i] = x5200;

			// y from 2*i from min to max via center
                        POT_input[2 * i + 1] = y5200;
                }
#endif
        }

	// Override key with chatboard for now!
	int key_break = 0;
	static int ctrl = 0;
	int ch = -1; //TODO (chatboard)pollKbdKey();
	if (ch >=0)
	{
//		debugText = debugTextBuffer;
//		sprintf(debugTextBuffer,"Key: %d\n", key_code);

		if (ch == 0x7f)
			key_break = 1;
		else if (ch == 0x7e)
			ctrl = 1;
		else
		{
			key_code = ch;

			if (ctrl)
			{
				ctrl = 0;
				key_code |= AKEY_CTRL;
			}
		}
	}	
	else if (key_code == AKEY_BREAK)
	{
		key_break = 1;
	}

	switch(operation)
	{
	case AKEY_FASTFORWARDON:
		setFastForward(1);
		break;
	case AKEY_FASTFORWARDOFF:
		setFastForward(0);
		break;
	case AKEY_FASTFORWARDTOGGLE:
		setFastForward(!fastForward);
		while (joyPressed());
		break;
	case AKEY_COLDSTART:
		menuClean();
		Coldstart();
		break;
	case AKEY_WARMSTART:
		menuClean();
		Aprint("Warmstart\n");
		Warmstart();
		Aprint("Warmstart done\n");
		break;

	case  AKEY_UI:
		if (fastForward) //Hack...
		{
			setFastForward(0);
			break;
		}
		setFastForward(0);
		Sound_Pause();
		menuWidth = 0;
		menuHeight = 0;
		ui();
		while (joyPressed()) usleep(1);
		menuClean();
		menuWidth = 0;
		menuHeight = 0;
		Sound_Continue();
		break;
	}
}

