#include "menu_ausbkeyboard.h"
#include <iostream>

extern "C"
{
#include "atari.h"
#include "menu_keymap.h"
#include "atari_gp32.h"
}

#include <devicemanager.h>

static Keyboard * getKeyboard(int pos)
{
	static std::vector<Keyboard *> sticks;
	if (sticks.size()<pos)
		sticks.resize(pos*2);
	Keyboard * stick = sticks[pos-1];
	if (!stick)
		stick = sticks[pos-1] = &DeviceManager::instance().acquireKeyboard();

	return stick;
}

AUSBKeyboardMenu::AUSBKeyboardMenu(int driver, const std::deque<std::string> & tokens):
	AMenu(driver),
	d_keyboard(0)
{
	d_keyboard = getKeyboard(1);
}

AUSBKeyboardMenu::~AUSBKeyboardMenu()
{
	//DeviceManager::instance().release(d_keyboard);
}

void AUSBKeyboardMenu::frame(int keysPressed)
{
	clean();

	int key = d_keyboard->lastPressedNoModifier();
	if (key>0)
	{
		std::cout << "Pressed:" << key << std::endl;
		int ucKey = d_keyboard->toUnicode(key);
		std::cout << "uckey:" << ucKey << std::endl;
		if (true) // (ucKey&0xff) == ucKey) XXX - its not unicode...
		{
			unsigned char asciiKey = ucKey&0xff; //FIXME
			std::cout << "ascii:" << asciiKey << std::endl;
			switch (asciiKey)
			{
				case 'a': d_keyPressed = AKEY_a; break;
				case 'b': d_keyPressed = AKEY_b; break;
				case 'c': d_keyPressed = AKEY_c; break;
				case 'd': d_keyPressed = AKEY_d; break;
				case 'e': d_keyPressed = AKEY_e; break;
				case 'f': d_keyPressed = AKEY_f; break;
				case 'g': d_keyPressed = AKEY_g; break;
				case 'h': d_keyPressed = AKEY_h; break;
				case 'i': d_keyPressed = AKEY_i; break;
				case 'j': d_keyPressed = AKEY_j; break;
				case 'k': d_keyPressed = AKEY_k; break;
				case 'l': d_keyPressed = AKEY_l; break;
				case 'm': d_keyPressed = AKEY_m; break;
				case 'n': d_keyPressed = AKEY_n; break;
				case 'o': d_keyPressed = AKEY_o; break;
				case 'p': d_keyPressed = AKEY_p; break;
				case 'q': d_keyPressed = AKEY_q; break;
				case 'r': d_keyPressed = AKEY_r; break;
				case 's': d_keyPressed = AKEY_s; break;
				case 't': d_keyPressed = AKEY_t; break;
				case 'u': d_keyPressed = AKEY_u; break;
				case 'v': d_keyPressed = AKEY_v; break;
				case 'w': d_keyPressed = AKEY_w; break;
				case 'x': d_keyPressed = AKEY_x; break;
				case 'y': d_keyPressed = AKEY_y; break;
				case 'z': d_keyPressed = AKEY_z; break;	

				case 'A': d_keyPressed = AKEY_A; break;
				case 'B': d_keyPressed = AKEY_B; break;
				case 'C': d_keyPressed = AKEY_C; break;
				case 'D': d_keyPressed = AKEY_D; break;
				case 'E': d_keyPressed = AKEY_E; break;
				case 'F': d_keyPressed = AKEY_F; break;
				case 'G': d_keyPressed = AKEY_G; break;
				case 'H': d_keyPressed = AKEY_H; break;
				case 'I': d_keyPressed = AKEY_I; break;
				case 'J': d_keyPressed = AKEY_J; break;
				case 'K': d_keyPressed = AKEY_K; break;
				case 'L': d_keyPressed = AKEY_L; break;
				case 'M': d_keyPressed = AKEY_M; break;
				case 'N': d_keyPressed = AKEY_N; break;
				case 'O': d_keyPressed = AKEY_O; break;
				case 'P': d_keyPressed = AKEY_P; break;
				case 'Q': d_keyPressed = AKEY_Q; break;
				case 'R': d_keyPressed = AKEY_R; break;
				case 'S': d_keyPressed = AKEY_S; break;
				case 'T': d_keyPressed = AKEY_T; break;
				case 'U': d_keyPressed = AKEY_U; break;
				case 'V': d_keyPressed = AKEY_V; break;
				case 'W': d_keyPressed = AKEY_W; break;
				case 'X': d_keyPressed = AKEY_X; break;
				case 'Y': d_keyPressed = AKEY_Y; break;
				case 'Z': d_keyPressed = AKEY_Z; break;

				case '1': d_keyPressed = AKEY_1; break;
				case '2': d_keyPressed = AKEY_2; break;
				case '3': d_keyPressed = AKEY_3; break;
				case '4': d_keyPressed = AKEY_4; break;
				case '5': d_keyPressed = AKEY_5; break;
				case '6': d_keyPressed = AKEY_6; break;
				case '7': d_keyPressed = AKEY_7; break;
				case '8': d_keyPressed = AKEY_8; break;
				case '9': d_keyPressed = AKEY_9; break;
				case '0': d_keyPressed = AKEY_0; break;

				case ' ': d_keyPressed = AKEY_SPACE; break;
				case '!': d_keyPressed = AKEY_EXCLAMATION; break;
				case '"': d_keyPressed = AKEY_DBLQUOTE; break;
				case '\'': d_keyPressed = AKEY_QUOTE; break;
				case '$': d_keyPressed = AKEY_DOLLAR; break;
				case '%': d_keyPressed = AKEY_PERCENT; break;
				case '*': d_keyPressed = AKEY_ASTERISK; break;
				case '/': d_keyPressed = AKEY_SLASH; break;
				case '#': d_keyPressed = AKEY_HASH; break;
				case '&': d_keyPressed = AKEY_AMPERSAND; break;
				case '<': d_keyPressed = AKEY_LESS; break;
				case '>': d_keyPressed = AKEY_GREATER; break;
				case '=': d_keyPressed = AKEY_EQUAL; break;
				case '+': d_keyPressed = AKEY_PLUS; break;
				case '-': d_keyPressed = AKEY_MINUS; break;
				case '@': d_keyPressed = AKEY_AT; break;
				case '[': d_keyPressed = AKEY_BRACKETLEFT; break;
				case ']': d_keyPressed = AKEY_BRACKETRIGHT; break;
				case '(': d_keyPressed = AKEY_PARENLEFT; break;
				case ')': d_keyPressed = AKEY_PARENRIGHT; break;
				case ',': d_keyPressed = AKEY_COMMA; break;
				case '.': d_keyPressed = AKEY_FULLSTOP; break;
				case ';': d_keyPressed = AKEY_SEMICOLON; break;
				case ':': d_keyPressed = AKEY_COLON; break;
				case '?': d_keyPressed = AKEY_QUESTION; break;
				case '^': d_keyPressed = AKEY_CARET; break; // CIRCUMFLEX?
				case '_': d_keyPressed = AKEY_UNDERSCORE; break;
				case '|': d_keyPressed = AKEY_BAR; break;
			}

			switch (ucKey)
			{
				case 513: d_keyPressed = AKEY_RETURN; break;
				case 127: d_keyPressed = AKEY_BACKSPACE; break;
				case 27: d_keyPressed = AKEY_ESCAPE; break;
				case 1539: d_keyPressed = AKEY_UP; break;
				case 1536: d_keyPressed = AKEY_DOWN; break;
				case 1537: d_keyPressed = AKEY_LEFT; break;
				case 1538: d_keyPressed = AKEY_RIGHT; break;
			}
		}
	}
}

void AUSBKeyboardMenu::clean()
{
	d_keyPressed = AKEY_NONE;
	d_operPressed = 0;
	d_consolPressed = 0;
	d_shiftPressed = 0;
	d_joyPressed = 0;
	d_5200Pressed = 0;
}

