#include "menu_apandorakeyboard.h"
#include <iostream>
#include <stdexcept>

extern "C"
{
#include "atari.h"
#include "menu_keymap.h"
#include "atari_gp32.h"
#include <fcntl.h>
#include <unistd.h>
#include <string.h>
}

APandoraKeyboardMenu::APandoraKeyboardMenu(int driver, const std::deque<std::string> & tokens):
	AMenu(driver),
	d_pandoraKeys(-1),
	d_pandoraControls(-1)
{
	int i=0;
	char name[256];
	int fd=-1;
	for(;;++i)
	{
		sprintf(name, "/dev/input/event%i", i);
		fd = open(name, O_RDONLY|O_NONBLOCK);
		if (fd < 0) break; /* no more devices */
			ioctl(fd, EVIOCGNAME(sizeof(name)), name);
		if (strcmp(name, "keypad") == 0)
		{
			d_pandoraKeys = fd; /* found the buttons! */
			continue;
		}
		else if (strcmp(name, "gpio-keys") == 0)
		{
			d_pandoraControls = fd; /* found the buttons! */
			continue;
		}
		close(fd); /* we don't need this device */
	}
	if (d_pandoraKeys==-1 || d_pandoraControls==-1)
		throw std::runtime_error("keypad event input device missing!");
}

APandoraKeyboardMenu::~APandoraKeyboardMenu()
{
	close(d_pandoraKeys);
	close(d_pandoraControls);
}

int
APandoraKeyboardMenu::pressed(int key) const
{
	int element = key/8;
	int bit = 1<<(key%8);
	return ((d_keySnap[element]&bit)?1:0);
}

int
APandoraKeyboardMenu::controlPressed(int key) const
{
	int element = key/8;
	int bit = 1<<(key%8);
	return ((d_controlSnap[element]&bit)?1:0);
}

void APandoraKeyboardMenu::frame(int keysPressed)
{
	clean();

	memset(&d_keySnap,0,sizeof(d_keySnap));
	ioctl(d_pandoraKeys, EVIOCGKEY(sizeof(d_keySnap)), d_keySnap);

	memset(&d_controlSnap,0,sizeof(d_controlSnap));
	ioctl(d_pandoraControls, EVIOCGKEY(sizeof(d_controlSnap)), d_controlSnap);

	/*struct input_event ev;
	while (-1!=read(d_pandoraKeys, &ev, sizeof(ev)));
	while (-1!=read(d_pandoraControls, &ev, sizeof(ev)));
	*/

	long res = 0;

	int ctrlPressed = 0;
	if (controlPressed(KEY_RIGHTCTRL)) ctrlPressed=1;

	int fnPressed = 0;
	if (pressed(KEY_FN)) fnPressed=1;
	if (controlPressed(KEY_LEFT)&&fnPressed) d_keyPressed=AKEY_LEFT;
	if (controlPressed(KEY_RIGHT)&&fnPressed) d_keyPressed=AKEY_RIGHT;
	if (controlPressed(KEY_UP)&&fnPressed) d_keyPressed=AKEY_UP;
	if (controlPressed(KEY_DOWN)&&fnPressed) d_keyPressed=AKEY_DOWN;

	if (pressed(KEY_LEFTSHIFT)) d_shiftPressed=1;

	//fprintf(stderr,"%d %d\n",d_shiftPressed, pressed(KEY_A));

	if (pressed(KEY_A)&&d_shiftPressed) d_keyPressed=AKEY_A;
	if (pressed(KEY_B)&&d_shiftPressed) d_keyPressed=AKEY_B;
	if (pressed(KEY_C)&&d_shiftPressed) d_keyPressed=AKEY_C;
	if (pressed(KEY_D)&&d_shiftPressed) d_keyPressed=AKEY_D;
	if (pressed(KEY_E)&&d_shiftPressed) d_keyPressed=AKEY_E;
	if (pressed(KEY_F)&&d_shiftPressed) d_keyPressed=AKEY_F;
	if (pressed(KEY_G)&&d_shiftPressed) d_keyPressed=AKEY_G;
	if (pressed(KEY_H)&&d_shiftPressed) d_keyPressed=AKEY_H;
	if (pressed(KEY_I)&&d_shiftPressed) d_keyPressed=AKEY_I;
	if (pressed(KEY_J)&&d_shiftPressed) d_keyPressed=AKEY_J;
	if (pressed(KEY_K)&&d_shiftPressed) d_keyPressed=AKEY_K;
	if (pressed(KEY_L)&&d_shiftPressed) d_keyPressed=AKEY_L;
	if (pressed(KEY_M)&&d_shiftPressed) d_keyPressed=AKEY_M;
	if (pressed(KEY_N)&&d_shiftPressed) d_keyPressed=AKEY_N;
	if (pressed(KEY_O)&&d_shiftPressed) d_keyPressed=AKEY_O;
	if (pressed(KEY_P)&&d_shiftPressed) d_keyPressed=AKEY_P;
	if (pressed(KEY_Q)&&d_shiftPressed) d_keyPressed=AKEY_Q;
	if (pressed(KEY_R)&&d_shiftPressed) d_keyPressed=AKEY_R;
	if (pressed(KEY_S)&&d_shiftPressed) d_keyPressed=AKEY_S;
	if (pressed(KEY_T)&&d_shiftPressed) d_keyPressed=AKEY_T;
	if (pressed(KEY_U)&&d_shiftPressed) d_keyPressed=AKEY_U;
	if (pressed(KEY_V)&&d_shiftPressed) d_keyPressed=AKEY_V;
	if (pressed(KEY_W)&&d_shiftPressed) d_keyPressed=AKEY_W;
	if (pressed(KEY_X)&&d_shiftPressed) d_keyPressed=AKEY_X;
	if (pressed(KEY_Y)&&d_shiftPressed) d_keyPressed=AKEY_Y;
	if (pressed(KEY_Z)&&d_shiftPressed) d_keyPressed=AKEY_Z;

	if (pressed(KEY_A)&&!d_shiftPressed) d_keyPressed=AKEY_a;
	if (pressed(KEY_B)&&!d_shiftPressed) d_keyPressed=AKEY_b;
	if (pressed(KEY_C)&&!d_shiftPressed) d_keyPressed=AKEY_c;
	if (pressed(KEY_D)&&!d_shiftPressed) d_keyPressed=AKEY_d;
	if (pressed(KEY_E)&&!d_shiftPressed) d_keyPressed=AKEY_e;
	if (pressed(KEY_F)&&!d_shiftPressed) d_keyPressed=AKEY_f;
	if (pressed(KEY_G)&&!d_shiftPressed) d_keyPressed=AKEY_g;
	if (pressed(KEY_H)&&!d_shiftPressed) d_keyPressed=AKEY_h;
	if (pressed(KEY_I)&&!d_shiftPressed) d_keyPressed=AKEY_i;
	if (pressed(KEY_J)&&!d_shiftPressed) d_keyPressed=AKEY_j;
	if (pressed(KEY_K)&&!d_shiftPressed) d_keyPressed=AKEY_k;
	if (pressed(KEY_L)&&!d_shiftPressed) d_keyPressed=AKEY_l;
	if (pressed(KEY_M)&&!d_shiftPressed) d_keyPressed=AKEY_m;
	if (pressed(KEY_N)&&!d_shiftPressed) d_keyPressed=AKEY_n;
	if (pressed(KEY_O)&&!d_shiftPressed) d_keyPressed=AKEY_o;
	if (pressed(KEY_P)&&!d_shiftPressed) d_keyPressed=AKEY_p;
	if (pressed(KEY_Q)&&!d_shiftPressed) d_keyPressed=AKEY_q;
	if (pressed(KEY_R)&&!d_shiftPressed) d_keyPressed=AKEY_r;
	if (pressed(KEY_S)&&!d_shiftPressed) d_keyPressed=AKEY_s;
	if (pressed(KEY_T)&&!d_shiftPressed) d_keyPressed=AKEY_t;
	if (pressed(KEY_U)&&!d_shiftPressed) d_keyPressed=AKEY_u;
	if (pressed(KEY_V)&&!d_shiftPressed) d_keyPressed=AKEY_v;
	if (pressed(KEY_W)&&!d_shiftPressed) d_keyPressed=AKEY_w;
	if (pressed(KEY_X)&&!d_shiftPressed) d_keyPressed=AKEY_x;
	if (pressed(KEY_Y)&&!d_shiftPressed) d_keyPressed=AKEY_y;
	if (pressed(KEY_Z)&&!d_shiftPressed) d_keyPressed=AKEY_z;

	if (pressed(KEY_0)&&!d_shiftPressed) d_keyPressed=AKEY_0;
	if (pressed(KEY_1)&&!d_shiftPressed) d_keyPressed=AKEY_1;
	if (pressed(KEY_2)&&!d_shiftPressed) d_keyPressed=AKEY_2;
	if (pressed(KEY_3)&&!d_shiftPressed) d_keyPressed=AKEY_3;
	if (pressed(KEY_4)&&!d_shiftPressed) d_keyPressed=AKEY_4;
	if (pressed(KEY_5)&&!d_shiftPressed) d_keyPressed=AKEY_5;
	if (pressed(KEY_6)&&!d_shiftPressed) d_keyPressed=AKEY_6;
	if (pressed(KEY_7)&&!d_shiftPressed) d_keyPressed=AKEY_7;
	if (pressed(KEY_8)&&!d_shiftPressed) d_keyPressed=AKEY_8;
	if (pressed(KEY_9)&&!d_shiftPressed) d_keyPressed=AKEY_9;

	if (pressed(KEY_0)&&d_shiftPressed) d_keyPressed=AKEY_BRACKETRIGHT;
	//if (pressed(KEY_1)&&d_shiftPressed) d_keyPressed=AKEY_WTF;
	//if (pressed(KEY_2)&&d_shiftPressed) d_keyPressed=AKEY_BRACELEFT;
	//if (pressed(KEY_3)&&d_shiftPressed) d_keyPressed=AKEY_BRACERIGHT;
	//if (pressed(KEY_4)&&d_shiftPressed) d_keyPressed=AKEY_TILDE;
	if (pressed(KEY_5)&&d_shiftPressed) d_keyPressed=AKEY_PERCENT;
	if (pressed(KEY_6)&&d_shiftPressed) d_keyPressed=AKEY_CARET;
	if (pressed(KEY_7)&&d_shiftPressed) d_keyPressed=AKEY_AMPERSAND;
	if (pressed(KEY_8)&&d_shiftPressed) d_keyPressed=AKEY_ASTERISK;
	if (pressed(KEY_9)&&d_shiftPressed) d_keyPressed=AKEY_BRACKETLEFT;

	if (pressed(KEY_F1)) d_keyPressed=AKEY_HELP;
	if (pressed(KEY_F2)) d_consolPressed=CONSOL_START;
	if (pressed(KEY_F3)) d_consolPressed=CONSOL_SELECT;
	if (pressed(KEY_F4)) d_consolPressed=CONSOL_OPTION;
	if (pressed(KEY_F5)) d_operPressed=AKEY_WARMSTART;
	if (pressed(KEY_F6)) d_operPressed=AKEY_COLDSTART;

	if (pressed(KEY_SPACE)) d_keyPressed=AKEY_SPACE;

	if (pressed(KEY_ENTER)) d_keyPressed=AKEY_RETURN;

	if (pressed(KEY_BACKSPACE)&&!d_shiftPressed) d_keyPressed=AKEY_BACKSPACE;
	if (pressed(KEY_BACKSPACE)&&d_shiftPressed) d_keyPressed=AKEY_DELETE_CHAR;

	if (pressed(KEY_APOSTROPHE)) d_keyPressed=AKEY_QUOTE;
	if (pressed(KEY_F14)) d_keyPressed=AKEY_BAR;
	if (pressed(KEY_BACKSLASH)) d_keyPressed=AKEY_BACKSLASH;
	if (pressed(KEY_KPMINUS)) d_keyPressed=AKEY_MINUS;
	if (pressed(KEY_LEFTBRACE)) d_keyPressed=AKEY_PARENLEFT;
	if (pressed(KEY_KPPLUS)) d_keyPressed=AKEY_PLUS;
	if (pressed(KEY_EQUAL)) d_keyPressed=AKEY_EQUAL;
	if (pressed(KEY_F13)) d_keyPressed=AKEY_QUOTE;
	if (pressed(KEY_F22)) d_keyPressed=AKEY_DOLLAR;
	if (pressed(KEY_F12)) d_keyPressed=AKEY_BREAK;
	if (pressed(KEY_ESC)) d_keyPressed=AKEY_ESCAPE;
	if (pressed(KEY_RIGHTBRACE)) d_keyPressed=AKEY_PARENRIGHT;
	if (pressed(KEY_F19)) d_keyPressed=AKEY_DBLQUOTE;
	if (pressed(KEY_F17)) d_keyPressed=AKEY_EXCLAMATION;
	if (pressed(KEY_F16)) d_keyPressed=AKEY_HASH;
	if (pressed(KEY_F20)) d_keyPressed=AKEY_AT;
	if (pressed(KEY_QUESTION)) d_keyPressed=AKEY_QUESTION;
	if (pressed(KEY_F15)) d_keyPressed=AKEY_UNDERSCORE;
	if (pressed(KEY_SLASH)) d_keyPressed=AKEY_SLASH;
	if (pressed(KEY_TAB)) d_keyPressed=AKEY_TAB;
	if (pressed(KEY_INSERT)) d_keyPressed=AKEY_INSERT_CHAR;
	if (pressed(KEY_F21)) d_keyPressed=AKEY_COLON;
	if (pressed(KEY_CAPSLOCK)) d_keyPressed=AKEY_CAPSLOCK;
	if (pressed(KEY_SEMICOLON)) d_keyPressed=AKEY_SEMICOLON;


	if (pressed(KEY_DOT)&&!d_shiftPressed) d_keyPressed=AKEY_FULLSTOP;
	if (pressed(KEY_COMMA)&&!d_shiftPressed) d_keyPressed=AKEY_COMMA;


	if (pressed(KEY_DOT)&&d_shiftPressed) d_keyPressed=AKEY_LESS;
	if (pressed(KEY_COMMA)&&d_shiftPressed) d_keyPressed=AKEY_GREATER;

	if (ctrlPressed) d_keyPressed |=AKEY_CTRL;
}

void APandoraKeyboardMenu::clean()
{
	d_keyPressed = AKEY_NONE;
	d_operPressed = 0;
	d_consolPressed = 0;
	d_shiftPressed = 0;
	d_joyPressed = 0;
	d_5200Pressed = 0;
}

