#ifndef NATY
/*
  GP2X minimal library v0.7 by rlyeh, 2005.
 
  + GP2X video library with double buffering.
  + GP2X soundring buffer library with double buffering.
  + GP2X timer library.
  + GP2X joystick library.
 
  Thanks to Squidge, Robster, snaff and NK, for the help & previous work! :-)
  Feel free to
 
  License
  =======
 
  Free for non-commercial projects (it would be nice receiving a mail from you).
  Other cases, ask me first.
 
  GamePark Holdings is not allowed to use this library and/or use parts from it.

  (MW:I confirmed with Rlyeh on IRC I could use this in Atari800, which is GPL)
 
  What's new
  ==========
 
  0.7: sound volume added; rewritten timer; faster init code; fixed launch code when exiting
 
  0.6: timer added; palette handling improved and fixed ; sound pause added.
 
  0.5: sound patched for real stereo output (by using NK's solution); better init code.
 
  0.4: lots of cleanups; sound is threaded and double buffered now; 8 bpp video support; better exiting code.
 
  0.3: shorter library; improved joystick diagonal detection.
 
  0.2: better code layout; public release.
 
  0.1: beta release

  ---------------------------
   Messed up by Mark Watson for atari800!
*/

#include "gp2xminlib.h"

#include <unistd.h>
#include <fcntl.h>
#include <sys/mman.h>
#include <sys/ioctl.h>
#include <sys/soundcard.h>
#include <linux/fb.h>
#include <time.h>
#include <stdio.h>


enum  { GP2X_UP=0x1,       GP2X_LEFT=0x4,       GP2X_DOWN=0x10,  GP2X_RIGHT=0x40,
        GP2X_START=1<<8,   GP2X_SELECT=1<<9,    GP2X_L=1<<10,    GP2X_R=1<<11,
        GP2X_A=1<<12,      GP2X_B=1<<13,        GP2X_X=1<<14,    GP2X_Y=1<<15,
        GP2X_VOL_DOWN=1<<22, GP2X_VOL_UP=1<<23, GP2X_PUSH=1<<27, };

  unsigned char  *gp2x_screen8;
  unsigned int gp2x_bpp=0;
  unsigned long   gp2x_dev[8]={-1,-1,-1,-1,-1,-1,-1,-1}, gp2x_physvram[4], *gp2x_memregl;
  volatile unsigned long   gp2x_sound_pause=0, gp2x_sound=0, gp2x_sound_bits=0;
  unsigned short *gp2x_memregs, *gp2x_screen15, *gp2x_logvram15[2];
  unsigned short gp2x_sound_buffer16[8192];
  unsigned char gp2x_sound_buffer8[8192];
  volatile unsigned short  gp2x_palette[512];

void gp2x_video_flip(void)
{
  unsigned long address=gp2x_physvram[gp2x_physvram[3]];
 
  gp2x_screen15=gp2x_logvram15[gp2x_physvram[3]^=1];
  gp2x_screen8=(unsigned char *)gp2x_screen15;
 
  gp2x_memregs[0x290E>>1]=(unsigned short)(address & 0xFFFF);
  gp2x_memregs[0x2910>>1]=(unsigned short)(address >> 16);
  gp2x_memregs[0x2912>>1]=(unsigned short)(address & 0xFFFF);
  gp2x_memregs[0x2914>>1]=(unsigned short)(address >> 16);
}
 
void gp2x_video_setpalette(void)
{
  unsigned short *g=(unsigned short *)gp2x_palette; int i=512;
  gp2x_memregs[0x2958>>1]=0;                                                     
  while(i--) gp2x_memregs[0x295A>>1]=*g++;
}

void gp2x_fill_rect(unsigned short x, unsigned short y, unsigned short width, unsigned short height, unsigned short col)
{
	if (x+width > 320)
		width = 320-x;
	if (y+height > 240)
		height = 240-y;
	if (gp2x_bpp == 8)
	{
		unsigned char * surface = gp2x_screen8;
		surface +=y*320 + x;
		int j=0;
		for (j=0; j<height; ++j)
		{
			int i=0;
			for (i=0;i<width; ++i)
			{
				surface[i] = col;
			}
			surface +=320;
		}
	}
	else
	{
		unsigned short * surface = gp2x_screen15;
		surface +=y*320 + x;
		int j=0;
		for (j=0; j<height; ++j)
		{
			int i=0;
			for (i=0;i<width; ++i)
			{
				surface[i] = col;
			}
			surface +=320;
		}
	}
}

void gp2x_video_plot(unsigned short x, unsigned short y, unsigned short col)
{
	if (gp2x_bpp == 8)
	{
		gp2x_screen8[x+y*320] = col;
	}
	else
	{
		gp2x_screen15[x+y*320] = col;
	}
}

unsigned long gp2x_joystick_read(void)
{
  unsigned long value=(gp2x_memregs[0x1198>>1] & 0x00FF);

/*  if(value==0xFD) value=0xFA;
  if(value==0xF7) value=0xEB;
  if(value==0xDF) value=0xAF;
  if(value==0x7F) value=0xBE;
 */
  unsigned long res =  ~((gp2x_memregs[0x1184>>1] & 0xFF00) | value | (gp2x_memregs[0x1186>>1] << 16));

  static int volumelevel = 90/2;
  static int setvolumelevel = 255;
  if (res&GP2X_VOL_UP && volumelevel<89)
  {
    volumelevel++;
  }
  if (res&GP2X_VOL_DOWN && volumelevel>1)
  {
    volumelevel--;
  }
  if (setvolumelevel != volumelevel)
  {
    int v = volumelevel|(volumelevel<<8);
    if (gp2x_dev[4]!=-1)
    {
      ioctl(gp2x_dev[4], MIXER_WRITE(SOUND_MIXER_PCM), &v);
      setvolumelevel = volumelevel;
    }
  }
  
  return res;
}


void gp2x_sound_play()
{
    audio_buf_info abi;
    while (ioctl(gp2x_dev[3], SNDCTL_DSP_GETOSPACE, &abi) || abi.fragments<1)
      usleep(1000);		     

    struct timespec ts;
    ts.tv_sec=0;
    ts.tv_nsec=10000000;
		    
    while (1)//abi.fragments--)
    {
      if (gp2x_sound_bits==8)
      {
       	while (!gp2x_sound_frame(0, (void *)gp2x_sound_buffer8, abi.fragsize/2)) nanosleep(&ts, 0);
	int i=0;
	int end = abi.fragsize/2;
	for (i=0; i!=end; ++i)
	{
		unsigned short out = (unsigned char)gp2x_sound_buffer8[i]; //signed - max 128
		//char * ptr = &gp2x_sound_buffer16[i];
		//ptr[0] = out&0xff;
		//ptr[1] = (out&0xff00)>>8;
		gp2x_sound_buffer16[i] = out<<7;

		int result = gp2x_sound_buffer16[i];
    static int min=1000000;
    static int min2=1000000;
    static int max=-1000000;
    static int max2=-1000000;
    static int ct=0;

    if (result<min) min=result;
    if (result>max) max=result;
    if (out<min2) min2=out;
    if (out>max2) max2=out;
    ++ct;
    if (ct>10000)
    {
	    ct = 0;
	    fprintf(stderr,"OUTmin:%d,%d max:%d,%d\n", min,min2,max,max2);
	    min = 1000000;
	    max = -min;
	    min2= 1000000;
	    max2 = -min2;
    }

	}
      }
      else
      {
        while (!gp2x_sound_frame(0, (void *)gp2x_sound_buffer16, abi.fragsize)) nanosleep(&ts, 0);	      
      }
      write(gp2x_dev[3], (void *)gp2x_sound_buffer16, abi.fragsize);
    }
}


void gp2x_deinit(void)
{
  close(gp2x_dev[0]);
  close(gp2x_dev[1]);
  close(gp2x_dev[2]);
  close(gp2x_dev[3]);
  close(gp2x_dev[4]);

  chdir("/usr/gp2x");
  execl("gp2xmenu",NULL);
}

void gp2x_reg_init()
{ 
  gp2x_dev[0] = gp2x_dev[1] = gp2x_dev[3] = -1;	
  gp2x_dev[2] = open("/dev/mem", O_RDWR);
  gp2x_memregs=(unsigned short *)mmap(0, 0x10000, PROT_READ|PROT_WRITE, MAP_SHARED, gp2x_dev[2], 0xc0000000);
  gp2x_memregl=(unsigned long *)(gp2x_memregs);
}

void gp2x_video_RGB_setscaling(int W, int H)
{
	float escalaw,escalah;
	int bpp=(gp2x_memregs[0x28DA>>1]>>9)&0x3;

	if(gp2x_memregs[0x2800>>1]&0x100) //TV-Out
	{
		escalaw=489.0; //RGB Horiz TV (PAL, NTSC)
		if (gp2x_memregs[0x2818>>1]  == 287) //PAL
			escalah=274.0; //RGB Vert TV PAL
		else if (gp2x_memregs[0x2818>>1]  == 239) //NTSC
			escalah=331.0; //RGB Vert TV NTSC
	}
	else //LCD
	{
		escalaw=1024.0; //RGB Horiz LCD
		escalah=320.0; //RGB Vert LCD
	}

	// scale horizontal
	gp2x_memregs[0x2906>>1]=(unsigned short)((float)escalaw *(W/320.0));
	// scale vertical
	gp2x_memregl[0x2908>>2]=(unsigned long)((float)escalah *bpp *(H/240.0));
}

void gp2x_video_init(int bpp)
{
struct fb_fix_screeninfo fixed_info;
gp2x_bpp = bpp;

if (gp2x_dev[0]==-1)
{
gp2x_dev[0] = open("/dev/fb0", O_RDWR);
gp2x_dev[1] = open("/dev/fb1", O_RDWR);
}
  
ioctl(gp2x_dev[gp2x_physvram[3]=0], FBIOGET_FSCREENINFO, &fixed_info);
gp2x_screen15=gp2x_logvram15[0]=(unsigned short *)mmap(0, 320*240*2, PROT_WRITE, MAP_SHARED, gp2x_dev[0], 0);
gp2x_screen8=(unsigned char *)(gp2x_screen15);
gp2x_physvram[0]=fixed_info.smem_start;

ioctl(gp2x_dev[1], FBIOGET_FSCREENINFO, &fixed_info);
gp2x_logvram15[1]=(unsigned short *)mmap(0, 320*240*2, PROT_WRITE, MAP_SHARED, gp2x_dev[1], 0);
gp2x_physvram[1]=fixed_info.smem_start;

gp2x_memregs[0x28DA>>1]=(((bpp+1)/8)<<9)|0xAB; /*8/15/16/24bpp...*/
gp2x_memregs[0x290C>>1]=320*((bpp+1)/8);       /*line width in bytes*/

gp2x_video_RGB_setscaling(320,240);
}

void gp2x_sound_init(int rate, int bits, int stereo)
{
gp2x_dev[3] = open("/dev/dsp", O_WRONLY);
gp2x_dev[4] = open("/dev/mixer", O_RDONLY);
gp2x_sound_bits = bits;

int bits16=16; //nothing else allowed!
ioctl(gp2x_dev[3], SNDCTL_DSP_SPEED,  &rate);
ioctl(gp2x_dev[3], SNDCTL_DSP_SETFMT, &bits16);
ioctl(gp2x_dev[3], SNDCTL_DSP_STEREO, &stereo);

int frags = 0xf0009;
ioctl(gp2x_dev[3], SNDCTL_DSP_SETFRAGMENT, &frags);

// fix for stereo I think
gp2x_memregs[0x0F16>>1] = 0x830a; usleep(100000);
gp2x_memregs[0x0F58>>1] = 0x100c; usleep(100000); 
}
#endif

