#ifndef NATY
/*
  GP2X minimal library v0.7 by rlyeh, 2005.
 
  + GP2X video library with double buffering.
  + GP2X soundring buffer library with double buffering.
  + GP2X timer library.
  + GP2X joystick library.
 
  Thanks to Squidge, Robster, snaff and NK, for the help & previous work! :-)
  Feel free to
 
  License
  =======
 
  Free for non-commercial projects (it would be nice receiving a mail from you).
  Other cases, ask me first.
 
  GamePark Holdings is not allowed to use this library and/or use parts from it.

  (MW:I confirmed with Rlyeh on IRC I could use this in Atari800, which is GPL)
 
  What's new
  ==========
 
  0.7: sound volume added; rewritten timer; faster init code; fixed launch code when exiting
 
  0.6: timer added; palette handling improved and fixed ; sound pause added.
 
  0.5: sound patched for real stereo output (by using NK's solution); better init code.
 
  0.4: lots of cleanups; sound is threaded and double buffered now; 8 bpp video support; better exiting code.
 
  0.3: shorter library; improved joystick diagonal detection.
 
  0.2: better code layout; public release.
 
  0.1: beta release

  ---------------------------
   Messed up by Mark Watson for atari800!
  ---------------------------
  Messed up even more to make it work on Pandora!!
*/

#include "gp2xminlib.h"

#include <unistd.h>
#include <fcntl.h>
#include <sys/mman.h>
#include <sys/ioctl.h>
#include <sys/soundcard.h>
#include <linux/fb.h>
#include <linux/omapfb.h>
#include <linux/input.h>
#include <time.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define FBIO_WAITFORVSYNC   _IOW('F', 0x20, u_int32_t)

enum  { GP2X_UP=0x1,       GP2X_LEFT=0x4,       GP2X_DOWN=0x10,  GP2X_RIGHT=0x40,
        GP2X_START=1<<8,   GP2X_SELECT=1<<9,    GP2X_L=1<<10,    GP2X_R=1<<11,
        GP2X_A=1<<12,      GP2X_B=1<<13,        GP2X_X=1<<14,    GP2X_Y=1<<15,
        GP2X_VOL_DOWN=1<<22, GP2X_VOL_UP=1<<23, GP2X_PUSH=1<<27, };

  unsigned char  *gp2x_physvram;
  unsigned char  *gp2x_screen8;
  unsigned int gp2x_bpp=0;
  int gp2x_width=0;
  int gp2x_height=0;
  int gp2x_buffers=0;
  int gp2x_dsp;
  int gp2x_video;
  int gp2x_videobg;
  int gp2x_keys;

  volatile unsigned long   gp2x_sound_pause=0, gp2x_sound=0, gp2x_sound_bits=0;
  unsigned short *gp2x_memregs, *gp2x_screen15;
  unsigned short gp2x_sound_buffer16[8192];
  unsigned char gp2x_sound_buffer8[8192];
  volatile struct fb_cmap gp2x_palette;

void gp2x_video_flip(void)
{
  static int buffer=0;

  int drawnBuffer = buffer++;
  if (buffer>=gp2x_buffers) buffer=0;

  // We need to start writing next frame to nextBuffer
  int framesize =  (gp2x_width*gp2x_height*((gp2x_bpp+7)/8));
  char unsigned * address = gp2x_physvram+framesize*buffer; 
  gp2x_screen15=(unsigned short *)address;
  gp2x_screen8=(unsigned char *)gp2x_screen15;

  // We should start displaying the frame we just finished drawing 
	struct fb_var_screeninfo fbvar;
	memset(&fbvar,0,sizeof(fbvar));
	ioctl(gp2x_video, FBIOGET_VSCREENINFO, &fbvar);
	fbvar.yoffset = gp2x_height*drawnBuffer;
	//fprintf(stderr,"dev:%d buffer:%d off:%d screenoff:%d/%d\n",gp2x_video,buffer,fbvar.yoffset,(gp2x_screen8-gp2x_physvram),framesize);
	ioctl(gp2x_video, FBIOPAN_DISPLAY, &fbvar);

	int arg = 0;
	ioctl(gp2x_video, FBIO_WAITFORVSYNC, &arg);
}

void gp2x_video_color8(int C, int R, int G, int B)
{
	gp2x_palette.red[C]=R*256;
	gp2x_palette.green[C]=G*256;
	gp2x_palette.blue[C]=B*256;
}
 
void gp2x_video_setpalette(void)
{
//struct fb_cmap {
//        __u32 start;                    /* First entry  */
//        __u32 len;                      /* Number of entries */
//        __u16 *red;                     /* Red values   */
//        __u16 *green;
//        __u16 *blue;
//        __u16 *transp;                  /* transparency, can be NULL */
//};
//#define FBIOPUTCMAP             0x4605
      ioctl(gp2x_video, FBIOPUTCMAP, &gp2x_palette);
}

void gp2x_fill_rect(unsigned short x, unsigned short y, unsigned short width, unsigned short height, unsigned short col)
{
	if (x+width > gp2x_width)
		width = gp2x_width-x;
	if (y+height > gp2x_height)
		height = gp2x_height-y;
	if (gp2x_bpp == 8)
	{
		unsigned char * surface = gp2x_screen8;
		surface +=y*gp2x_width + x;
		int j=0;
		for (j=0; j<height; ++j)
		{
			int i=0;
			for (i=0;i<width; ++i)
			{
				surface[i] = col;
			}
			surface +=gp2x_width;
		}
	}
	else
	{
		unsigned short * surface = gp2x_screen15;
		surface +=y*gp2x_width + x;
		int j=0;
		for (j=0; j<height; ++j)
		{
			int i=0;
			for (i=0;i<width; ++i)
			{
				surface[i] = col;
			}
			surface +=gp2x_width;
		}
	}
}

void gp2x_video_plot(unsigned short x, unsigned short y, unsigned short col)
{
	if (gp2x_bpp == 8)
	{
		gp2x_screen8[x+y*gp2x_width] = col;
	}
	else
	{
		gp2x_screen15[x+y*gp2x_width] = col;
	}
}

char key_b[KEY_MAX/8 + 1];
int pressed(int key)
{
	int element = key/8;
	int bit = 1<<(key%8);
	return ((key_b[element]&bit)?1:0);
}

unsigned long gp2x_joystick_read(void)
{
        memset(key_b, 0, sizeof(key_b));
        ioctl(gp2x_keys, EVIOCGKEY(sizeof(key_b)), key_b);
	
	long res = 0;
	if (pressed(KEY_UP)) res|=GP2X_UP;
	if (pressed(KEY_DOWN)) res|=GP2X_DOWN;
	if (pressed(KEY_LEFT)) res|=GP2X_LEFT;
	if (pressed(KEY_RIGHT)) res|=GP2X_RIGHT;
	if (pressed(KEY_LEFTALT)) res|=GP2X_START;
	if (pressed(KEY_LEFTCTRL)) res|=GP2X_SELECT;
	if (pressed(KEY_RIGHTSHIFT)) res|=GP2X_L;
	if (pressed(KEY_RIGHTCTRL)) res|=GP2X_R;
	if (pressed(KEY_HOME)) res|=GP2X_A;
	if (pressed(KEY_END)) res|=GP2X_B;
	if (pressed(KEY_PAGEDOWN)) res|=GP2X_X;
	if (pressed(KEY_END)) res|=GP2X_Y;

	//fprintf(stderr, "keys:%d JOY:%x\n" , gp2x_keys,res);

  return res;
  // - evdev
/*enum  { GP2X_UP=0x1,       GP2X_LEFT=0x4,       GP2X_DOWN=0x10,  GP2X_RIGHT=0x40,
        GP2X_START=1<<8,   GP2X_SELECT=1<<9,    GP2X_L=1<<10,    GP2X_R=1<<11,
        GP2X_A=1<<12,      GP2X_B=1<<13,        GP2X_X=1<<14,    GP2X_Y=1<<15,
        GP2X_VOL_DOWN=1<<22, GP2X_VOL_UP=1<<23, GP2X_PUSH=1<<27, };
*/

/*KEY_UP, KEY_DOWN, KEY_LEFT, KEY_RIGHT, KEY_MENU (Pandora button), KEY_LEFTALT (Start), KEY_LEFTCTRL (Select), KEY_END (Y/North), KEY_HOME (A/East), KEY_PAGEDOWN (X/South), KEY_END (B/West), KEY_RIGHTSHIFT (Shoulder L), KEY_RIGHTCTRL (Shoulder R), KEY_KPPLUS (Shoulder L2), KEY_KPMINUS (Shoulder R2), KEY_COFFEE (Hold)*/
}


void gp2x_sound_play()
{
    audio_buf_info abi;
    while (ioctl(gp2x_dsp, SNDCTL_DSP_GETOSPACE, &abi) || abi.fragments<1)
      usleep(1000);		     

    struct timespec ts;
    ts.tv_sec=0;
    ts.tv_nsec=10000000;
		    
    while (1)//abi.fragments--)
    {
      if (gp2x_sound_bits==8)
      {
       	while (!gp2x_sound_frame(0, (void *)gp2x_sound_buffer8, abi.fragsize/2)) nanosleep(&ts, 0);
	int i=0;
	int end = abi.fragsize/2;
	for (i=0; i!=end; ++i)
	{
		unsigned short out = (unsigned char)gp2x_sound_buffer8[i]; //signed - max 128
		//char * ptr = &gp2x_sound_buffer16[i];
		//ptr[0] = out&0xff;
		//ptr[1] = (out&0xff00)>>8;
		gp2x_sound_buffer16[i] = out<<7;

		int result = gp2x_sound_buffer16[i];
    static int min=1000000;
    static int min2=1000000;
    static int max=-1000000;
    static int max2=-1000000;
    static int ct=0;

    if (result<min) min=result;
    if (result>max) max=result;
    if (out<min2) min2=out;
    if (out>max2) max2=out;
    ++ct;
    if (ct>10000)
    {
	    ct = 0;
	    fprintf(stderr,"OUTmin:%d,%d max:%d,%d\n", min,min2,max,max2);
	    min = 1000000;
	    max = -min;
	    min2= 1000000;
	    max2 = -min2;
    }

	}
      }
      else
      {
        while (!gp2x_sound_frame(0, (void *)gp2x_sound_buffer16, abi.fragsize)) nanosleep(&ts, 0);	      
      }
      write(gp2x_dsp, (void *)gp2x_sound_buffer16, abi.fragsize);
    }
}


void gp2x_deinit(void)
{
  close(gp2x_dsp);
  close(gp2x_video);
  close(gp2x_videobg);
  close(gp2x_keys);
}


int getKeys()
{
	int i=0;
	char name[256];
	int fd=-1;
	while(1)
	{
		sprintf(name, "/dev/input/event%i", i);
		fd = open(name, O_RDONLY);
		if (fd < 0) break; /* no more devices */
			ioctl(fd, EVIOCGNAME(sizeof(name)), name);
		if (strcmp(name, "gpio-keys") == 0)
			return fd; /* found the buttons! */
		close(fd); /* we don't need this device */
		++i;
	}
	return fd;
}

void gp2x_reg_init()
{ 
	int i=0;
/*	gp2x_videobg = open("/dev/fb0", O_RDWR);
	unsigned char *bg = (unsigned char *)mmap(0, 800*480*2, PROT_WRITE, MAP_SHARED, gp2x_videobg, 0);
	for (i=0;i!=(800*480*2); ++i) *bg=0;
exit(0);*/

  fprintf(stderr,"gp2x_reg_init\n");
  gp2x_dsp = -1;
  gp2x_video=-1;
  gp2x_videobg=-1;

	gp2x_palette.start=0;
	gp2x_palette.len=256;
	gp2x_palette.red=malloc(sizeof(__u16)*256);
	gp2x_palette.green=malloc(sizeof(__u16)*256);
	gp2x_palette.blue=malloc(sizeof(__u16)*256);
	gp2x_palette.transp=0;

  gp2x_keys=getKeys();

  fprintf(stderr,"gp2x_reg_init done Keys:%d\n",gp2x_keys);
}

void gp2x_video_init(int bpp, int width, int height, int buffers)
{
  fprintf(stderr,"gp2x_video_init\n");
  if (width==gp2x_width && height==gp2x_height && bpp==gp2x_bpp&&buffers==gp2x_buffers) return;
	if (gp2x_video!=-1)
	{
		munmap(gp2x_physvram,gp2x_width*gp2x_height*((gp2x_bpp+7)/8)*gp2x_buffers);
		close(gp2x_video);
		gp2x_video=-1;
	}

	gp2x_bpp = bpp;
	gp2x_buffers = buffers;
	gp2x_width = width;
	gp2x_height = height;

	// call fbset to set screen mode
	char buffer[256];
	char const * extra = "";
	//if (bpp==8) extra="-nonstd 3";
	//if (bpp==16) extra="-nonstd 0";
	int tempbpp=16;
	sprintf(buffer,"/usr/sbin/fbset -fb /dev/fb1 -g %d %d %d %d %d %s",width,height,width,height*buffers,tempbpp,extra);
  	fprintf(stderr,"gp2x_video_init running %s\n", buffer);
	system(buffer);
  	fprintf(stderr,"gp2x_video_init run %s\n", buffer);
	

	// set up the planes properly - looks to me like fbset is broken in linux kernel itself for 8-bit mode - I checked!
	// (Trivial to fix so not great...)
	{
		gp2x_video = open("/dev/fb0", O_RDWR);

		struct omapfb_plane_info pi;
		memset(&pi, 0, sizeof(pi));
		int ret = ioctl(gp2x_video, OMAPFB_QUERY_PLANE, &pi);

		struct omapfb_mem_info mi;
		memset(&mi, 0, sizeof(mi));
		ret = ioctl(gp2x_video, OMAPFB_QUERY_MEM, &mi);
		fprintf(stderr,"FB0 planeInfo res:%d pos_x:%d pos_y:%d enabled:%d channel_out:%d mirror:%d out_width:%d out_height:%d size:%d type:%d\n",ret,pi.pos_x,pi.pos_y,pi.enabled,pi.channel_out,pi.mirror,pi.out_width,pi.out_height,mi.size,mi.type);

		// Dump out the current info
		/*struct omapfb_plane_info {
			__u32 pos_x;
			__u32 pos_y;
			__u8  enabled;
			__u8  channel_out;
			__u8  mirror;
			__u8  reserved1;
			__u32 out_width;
			__u32 out_height;
			__u32 reserved2[12];
		};

		struct omapfb_mem_info {
			__u32 size;
			__u8  type;
			__u8  reserved[3];
		};*/

		close(gp2x_video);
	}

	{
		gp2x_video = open("/dev/fb1", O_RDWR);

		struct omapfb_plane_info pi;
		memset(&pi, 0, sizeof(pi));
		int ret = ioctl(gp2x_video, OMAPFB_QUERY_PLANE, &pi);

		struct omapfb_mem_info mi;
		memset(&mi, 0, sizeof(mi));
		ret = ioctl(gp2x_video, OMAPFB_QUERY_MEM, &mi);
		fprintf(stderr,"FB1 planeInfo res:%d pos_x:%d pos_y:%d enabled:%d channel_out:%d mirror:%d out_width:%d out_height:%d size:%d type:%d\n",ret,pi.pos_x,pi.pos_y,pi.enabled,pi.channel_out,pi.mirror,pi.out_width,pi.out_height,mi.size,mi.type);

		// Dump out the current info
		/*struct omapfb_plane_info {
			__u32 pos_x;
			__u32 pos_y;
			__u8  enabled;
			__u8  channel_out;
			__u8  mirror;
			__u8  reserved1;
			__u32 out_width;
			__u32 out_height;
			__u32 reserved2[12];
		};

		struct omapfb_mem_info {
			__u32 size;
			__u8  type;
			__u8  reserved[3];
		};*/

		close(gp2x_video);
	}


	/*{
		gp2x_video = open("/dev/fb1", O_RDWR);
		struct fb_var_screeninfo fbvar;
		memset(&fbvar,0,sizeof(fbvar));
		int res = ioctl(gp2x_video, FBIOGET_VSCREENINFO, &fbvar);
		fprintf(stderr,"screenInfo xres:%d yres;%d xresvirt:%d yresvirt:%d xoffset:%d yoffset:%d bpp:%d nonstd:%d",fbvar.xres,fbvar.yres,fbvar.xres_virtual,fbvar.yres_virtual,fbvar.xoffset,fbvar.yoffset,fbvar.bits_per_pixel,fbvar.nonstd);

		fbvar.xres = width;
		fbvar.yres = height;
		fbvar.xres_virtual = width;
		fbvar.yres_virtual = height * buffers;
		fbvar.xoffset = 0;
		fbvar.yoffset = 0;
		fbvar.bits_per_pixel = 0;
		fbvar.nonstd = 0; // can set YUV here on omapfb
		res = ioctl(gp2x_video, FBIOPUT_VSCREENINFO, &fbvar);

		close(gp2x_video);
	}*/
	
	{
		gp2x_video = open("/dev/fb1", O_RDWR);
		gp2x_physvram = (unsigned char *)mmap(0, width*height*((bpp+7)/8)*buffers, PROT_WRITE, MAP_SHARED, gp2x_video, 0);
		gp2x_screen15=(unsigned short *)(gp2x_physvram);
		gp2x_screen8=(unsigned char *)(gp2x_screen15);
	}
  fprintf(stderr,"gp2x_video_init_done\n");
}

void gp2x_sound_init(int rate, int bits, int stereo)
{
gp2x_dsp = open("/dev/dsp", O_WRONLY);
gp2x_sound_bits = bits;

int bits16=16; //nothing else allowed!
ioctl(gp2x_dsp, SNDCTL_DSP_SPEED,  &rate);
ioctl(gp2x_dsp, SNDCTL_DSP_SETFMT, &bits16);
ioctl(gp2x_dsp, SNDCTL_DSP_STEREO, &stereo);

int frags = 0xf0009;
ioctl(gp2x_dsp, SNDCTL_DSP_SETFRAGMENT, &frags);

// fix for stereo I think
//gp2x_memregs[0x0F16>>1] = 0x830a; usleep(100000);
//gp2x_memregs[0x0F58>>1] = 0x100c; usleep(100000); 
}
#endif

